@@ -1,11 +1,10 @@
 package Mojolicious::Plugin::ReCAPTCHAv2;
-# vim:syntax=perl:tabstop=4:number:noexpandtab:
-$Mojolicious::Plugin::ReCAPTCHAv2::VERSION = '0.2';
+$Mojolicious::Plugin::ReCAPTCHAv2::VERSION = '0.03';
 use Mojo::Base 'Mojolicious::Plugin';
 
 # ABSTRACT: use Googles "No CAPTCHA reCAPCTHA" (reCAPTCHA v2) service in Mojolicious apps
 
-use Mojo::JSON qw( true false );
+use Mojo::JSON qw();
 use Mojo::UserAgent qw();
 
 has conf => sub{ +{} };
@@ -13,7 +12,7 @@ has verification_errors => sub{ +[] };
 
 sub register {
 	my $plugin = shift;
-	my $app    = shift;
+	my $app	= shift;
 	my $conf   = shift || {};
 
 	die ref($plugin), ": need sitekey and secret!\n"
@@ -26,8 +25,8 @@ sub register {
 
 	$app->helper(
 		recaptcha_get_html => sub {
-			my $c         = shift;
-			my $language  = $_[0] ? shift : undef;
+			my $c		= shift;
+			my $language = $_[0] ? shift : undef;
 
 			my %data_attr = map { $_ => $plugin->conf->{$_} } grep { index( $_, 'api_' ) != 0 } keys %{ $plugin->conf };
 
@@ -42,26 +41,13 @@ sub register {
 				$hl = delete $data_attr{'language'};
 			}
 
-			my $output = '';
-			my $template = q|<script src="https://www.google.com/recaptcha/api.js?hl=<%= $hl %>" async defer></script>
-<div class="g-recaptcha"<% foreach my $k ( sort keys %{$attr} ) { %> data-<%= $k %>="<%= $attr->{$k} %>"<% } %>></div>|;
-
-			# Compatibility with Mojolicious < 5.0
-			if ( $c->can('render_to_string') ) {
-				$output = $c->render_to_string(
-					inline => $template,
-					hl     => $hl,
-					attr   => \%data_attr,
-				);
-			}
-			else {
-				$output = $c->render(
-					inline  => $template,
-					hl      => $hl,
-					attr    => \%data_attr,
-					partial => 1,
-				);
-			}
+			my $output = $c->render_to_string(
+				inline => q|<script src="https://www.google.com/recaptcha/api.js?hl=<%= $hl %>" async defer></script>
+<div class="g-recaptcha"<% foreach my $k ( keys %{$attr} ) { %> data-<%= $k %>="<%= $attr->{$k} %>"<% } %>></div>|,
+				hl	 => $hl,
+				attr   => \%data_attr,
+			);
+									 
 			return $output;
 		}
 	);
@@ -71,11 +57,11 @@ sub register {
 
 			my %verify_params = (
 				remoteip => $c->tx->remote_address,
-				response => ( $c->req->param('g-recaptcha-response') || '' ),
+				response => $c->req->param('g-recaptcha-response'),
 				secret   => $plugin->conf->{'secret'},
 			);
 
-			my $url     = $plugin->conf->{'api_url'};
+			my $url	    = $plugin->conf->{'api_url'};
 			my $timeout = $plugin->conf->{'api_timeout'};
 
 			my $ua = Mojo::UserAgent->new();
@@ -85,53 +71,29 @@ sub register {
 			$plugin->verification_errors([]);
 
 			# XXX async request?
-
-			my $tx = '';
-			# Backwards compatibility with older Mojolicious versions
-			if ( $ua->can('post_form') ) {
-				$tx = $ua->post_form( $url => \%verify_params );
-			}
-			else {
-				$tx = $ua->post( $url => form => \%verify_params );
-			}
-
+			my $tx = $ua->post( $url => form => \%verify_params );
 			if ( my $res = $tx->success ) {
 				my $json = '';
 				eval {
 					$json = Mojo::JSON::decode_json( $res->body );
 				};
-
-				# Compatibility with Mojo::JSON as of Mojolicious < 4.82
-				if ( defined($@) and index( $@, 'Mojo::JSON::decode_json' ) >= 0 ) {
-					eval {
-						my $obj = Mojo::JSON->new;
-						$json = $obj->decode( $res->body );
-					};
-				}
-
 				if ($@) {
 					$c->app->log->error( 'Decoding JSON response failed: ' . $@ );
 					$c->app->log->error( 'Request  was: ' . $tx->req->to_string );
 					$c->app->log->error( 'Response was: ' . $tx->res->to_string );
-					$plugin->verification_errors( ["x-unparseable-data-received"] );
-					return 0;
+					return -1;
 				}
-				unless ( $json->{'success'} == true ) {
+				unless ( $json->{'success'} ) {
 					$plugin->verification_errors( $json->{'error-codes'} // [] );
 				}
 				return $json->{'success'};
 
 			}
 			else {
-				my $err = $tx->error;
-				my $txt = 'Retrieving captcha verifcation failed';
-				$txt   .= ' (HTTP ' . $err->{'code'} . ')' if $err->{'code'};
-
-				$c->app->log->error( $txt . ': ' . $err->{'message'} );
+				$c->app->log->error( 'Retrieving captcha verifcation failed: HTTP ' . $res->code );
 				$c->app->log->error( 'Request  was: ' . $tx->req->to_string );
-				$plugin->verification_errors( ["x-http-communication-failed"] );
-
-				return 0;
+				$c->app->log->error( 'Response was: ' . $tx->res->to_string );
+				return -1;
 			}
 		}
 	);
@@ -158,17 +120,20 @@ Mojolicious::Plugin::ReCAPTCHAv2 - use Googles "No CAPTCHA reCAPCTHA" (reCAPTCHA
 
 =head1 VERSION
 
-version 0.2
+version 0.03
 
 =head1 SYNOPSIS
 
+B<WARNING> This module is considered experimental! Tests are mostly missing and it has not been used
+in production so far!
+
     use Mojolicious::Plugin::ReCAPTCHAv2;
 
     sub startup {
         my $self = shift;
 
-        $self->plugin('ReCAPTCHAv2', {
-            sitekey       => 'site-key-embedded-in-public-html',                 # required
+        $self->plugin('ReCAPTCHAv2', { 
+            site_key      => 'site-key-embedded-in-public-html',                 # required
             secret        => 'key-used-in-internal-verification-requests',       # required
             # api_timeout => 10,                                                 # optional
             # api_url     => 'https://www.google.com/recaptcha/api/siteverify',  # optional
@@ -186,16 +151,18 @@ version 0.2
     # now use stashed value in your HTML template, i.e.: <form..>...<% $captcha %>...</form>
 
     # on incoming request
-    if ( $app->recaptcha_verify ) {
+    my $verify = $app->recaptcha_verify;
+    if ( $verify < 0 ) {
+        # internal error - request failed or invalid json
+    }
+    elsif ( $verify > 0 ) {
         # success: probably human
-        ...
     }
     else {
         # fail: probably bot, but may also be a
         # processing error
-
         if ( my $err = $app->recaptcha_get_errors ) {
-            # processing failed, inspect error codes
+            # processing failed
             foreach my $e ( @{$err} ) {
                 ...
             }
@@ -271,23 +238,27 @@ L<Mojolicious::Plugin::ReCAPTCHAv2> makes the following helpers available:
 
 =head2 recaptcha_get_html
 
-Returns a HTML fragment with the widget code; you will probably want to put
-this in the stash, since it has to be inserted in your HTML form element
+Returns a HTML fragment with the widget codeM; you will probably want to put
+this in the stash, since it has to be inserted in your HTML form element 
 when processing the template.
 
 =head2 recaptcha_verify
 
 Call this helper when receiving the request from your website after the user
-submitted the form. Sends your secret, the response token from the request
-your received and the users IP to the reCAPTCHA server to verify the token.
+submitted the form.
 
 You should call this only once per incoming request.
 
-It will return either a C<true> or C<false> value:
+It will return one of the following values:
 
 =over 4
 
-=item C<false> (0)
+=item C<-1>
+
+Some internal error occured: the HTTP request failed or the result returned
+by the reCPATCHA servers is invalid JSON.
+
+=item C<0>
 
 The reCAPTCHA service could not verify that the Captcha was solved by a
 human; either because it was a bot or because of some processing error.
@@ -295,10 +266,10 @@ You should check for processing errors via C<recaptcha_get_errors>.
 You should not continue with processing your users request but probably
 re-display the form with an added error message.
 
-=item C<true> (1)
+=item C<1>
 
-The data is valid and the reCAPTCHA service believes that the challenge
-was solved by a human. You may proceed with processing the incoming request.
+The reCAPTCHA service believes that the challenge was solved by a human.
+You may proceed with processing the incoming request.
 
 =back
 
@@ -307,13 +278,13 @@ was solved by a human. You may proceed with processing the incoming request.
 This helper returns a reference to an array which may contain zero, one
 or more error codes.
 The array is reset on every call to C<recaptcha_verify>.
-The array can contain these official API error codes:
+The array can contain these error codes:
 
 =over 4
 
 =item C<missing-input-secret>
 
-The secret parameter is missing.
+The secret parameter is missing. 
 
 This should not happen, since registering the plugin requires a C<secret>
 configuration param which is then automatically included in the verification
@@ -341,23 +312,6 @@ Somebody tinkered with the request data somewhere.
 
 =back
 
-Additionally the following error codes may be encountered which are defined
-internally by this module. Note: these codes start with "x-" to 
-distinguish them from official error codes.
-
-=over 4
-
-=item C<x-http-communication-failed>
-
-Something went wrong while trying to talk to the reCAPTCHA server.
-
-=item C<x-unparseable-data-received>
-
-The http request was completed successfully but Mojo::JSON could not
-decode the response received from the reCAPTCHA server.
-
-=back
-
 =head1 SEE ALSO
 
 =over 4
